// SPDX-License-Identifier: MIT

pragma solidity ^0.8.19;

interface IGnosisSafe {
    enum Operation {
        Call,
        DelegateCall
    }

    function NAME() external view returns (string memory);

    function VERSION() external view returns (string memory);

    function nonce() external view returns (uint256);

    function domainSeparator() external view returns (bytes32);

    function signedMessages(bytes32) external view returns (uint256);

    function approvedHashes(address, bytes32) external view returns (uint256);

    function setup(
        address[] calldata _owners,
        uint256 _threshold,
        address to,
        bytes calldata data,
        address fallbackHandler,
        address paymentToken,
        uint256 payment,
        address payable paymentReceiver
    ) external;

    function execTransaction(
        address to,
        uint256 value,
        bytes calldata data,
        Operation operation,
        uint256 safeTxGas,
        uint256 baseGas,
        uint256 gasPrice,
        address gasToken,
        address payable refundReceiver,
        bytes calldata signatures
    ) external returns (bool success);

    function requiredTxGas(address to, uint256 value, bytes calldata data, Operation operation)
        external
        returns (uint256);

    function approveHash(bytes32 hashToApprove) external;

    function signMessage(bytes calldata _data) external;

    function isValidSignature(bytes calldata _data, bytes calldata _signature) external returns (bytes4);

    function getMessageHash(bytes memory message) external view returns (bytes32);

    function encodeTransactionData(
        address to,
        uint256 value,
        bytes memory data,
        Operation operation,
        uint256 safeTxGas,
        uint256 baseGas,
        uint256 gasPrice,
        address gasToken,
        address refundReceiver,
        uint256 _nonce
    ) external view returns (bytes memory);

    function getTransactionHash(
        address to,
        uint256 value,
        bytes memory data,
        Operation operation,
        uint256 safeTxGas,
        uint256 baseGas,
        uint256 gasPrice,
        address gasToken,
        address refundReceiver,
        uint256 _nonce
    ) external view returns (bytes32);
}
